@@ -12,13 +12,87 @@ import numpy as np
 from numpy import allclose, array_equal, diag, eye, linspace, zeros, isnan
 from numpy.linalg import cholesky, inv, LinAlgError
 import torch
-from scipy.optimize import minimize
 
 from causing import utils
 
 log = logging.getLogger(__name__)
 
 
+def directvec_alg(mx, my):
+    """algebraic direct effects vector column-wise
+    from direct effects matrices and id matrices"""
+
+    directy = my.T[~isnan(my.T)]
+    directx = mx.T[~isnan(mx.T)]
+    direct = np.concatenate((directy, directx), axis=0)
+
+    return direct
+
+
+def total_from_direct(direct, idx, idy, edx, edy):
+    """construct total effects vector from direct effects vector and id and ed matrices"""
+
+    mx, my = utils.directmat_alg(direct, idx, idy)
+    ex, ey = utils.total_effects_alg(mx, my, edx, edy)
+
+    effects = directvec_alg(ex, ey)
+
+    return effects
+
+
+def directmat(direct, idx, idy):
+    """automatic direct effects matrices column-wise
+    from direct effects vector and id matrices"""
+
+    # dimensions
+    ndim = idx.shape[0]
+    mdim = idx.shape[1]
+
+    # compute direct effects matrices
+    my = torch.DoubleTensor(zeros((ndim, ndim)))
+    mx = torch.DoubleTensor(zeros((ndim, mdim)))
+    k = 0
+    for i in range(ndim):
+        for j in range(ndim):
+            if idy[i, j] == 1:
+                my[i, j] = direct[k]
+                k += 1
+    for i in range(ndim):
+        for j in range(mdim):
+            if idx[i, j] == 1:
+                mx[i, j] = direct[k]
+                k += 1
+
+    return mx, my
+
+
+def directvec(mx, my, idx, idy):
+    """automatic direct effects vector column-wise
+    from direct effects matrices and id matrices"""
+
+    # dimensions
+    ndim = idx.shape[0]
+    mdim = idx.shape[1]
+    qydim = np.count_nonzero(idy)
+    qxdim = np.count_nonzero(idx)
+
+    # compute direct effects vector
+    direct = torch.DoubleTensor(zeros(qydim + qxdim))
+    k = 0
+    for i in range(ndim):
+        for j in range(ndim):
+            if idy[i, j] == 1:
+                direct[k] = my[i, j]
+                k += 1
+    for i in range(ndim):
+        for j in range(mdim):
+            if idx[i, j] == 1:
+                direct[k] = mx[i, j]
+                k += 1
+
+    return direct
+
+
 def sse_hess_num(mx, my, model_dat, alpha):
     """compute numeric Hessian of sse at given data and direct effects"""
 
@@ -34,7 +108,7 @@ def sse_hess_num(mx, my, model_dat, alpha):
         ssetikh = sse + alpha * direct.T @ direct
         return ssetikh
 
-    direct = utils.directvec(mx, my, model_dat["idx"], model_dat["idy"])
+    direct = directvec(mx, my, model_dat["idx"], model_dat["idy"])
     direct = direct.detach().numpy()
     hessian_num_func = nd.Hessian(lambda *direct: sse_orig_alg(direct, model_dat))
     hessian_num = hessian_num_func(direct)
@@ -224,7 +298,7 @@ def check_estimate_effects(model_dat, alpha, do_print=True):
     )
     ex_hat[model_dat["edx"] == 0] = float("NaN")
     ey_hat[model_dat["edy"] == 0] = float("NaN")
-    direct_hat = utils.directvec(mx_hat, my_hat, model_dat["idx"], model_dat["idy"])
+    direct_hat = directvec(mx_hat, my_hat, model_dat["idx"], model_dat["idy"])
 
     hessian_hat = sse_hess_alg(direct_hat, model_dat, alpha)
     check = check_hessian(hessian_hat)
@@ -248,7 +322,7 @@ def alpha_min_max(model_dat):
         * model_dat["ymcdat"]
         * diag(model_dat["selwei"]).reshape(-1, 1)
     )
-    direct_theo = utils.directvec_alg(model_dat["mx_theo"], model_dat["my_theo"])
+    direct_theo = directvec_alg(model_dat["mx_theo"], model_dat["my_theo"])
     directnorm = direct_theo.T @ direct_theo
     alpha_max_tmp = fraction * ymvar / directnorm
 
@@ -514,25 +588,6 @@ def estimate_effects(model_dat, estimate_input):
     return estimate_dat
 
 
-def estimate_biases(model_dat, ymdat):
-    """numerical optimize modification indicators for equations, one at a time"""
-
-    tau = model_dat["xdat"].shape[1]
-    biases = zeros(model_dat["ndim"])
-    biases_std = zeros(model_dat["ndim"])
-    for bias_ind in range(model_dat["ndim"]):
-        # compute biases
-        bias, hess_i, sse = optimize_biases(model_dat, bias_ind, ymdat)
-        biases[bias_ind] = bias
-
-        # compute biases_std
-        resvar = sse / (tau - 1)
-        bias_std = (2 * resvar * (1 / hess_i)) ** (1 / 2)
-        biases_std[bias_ind] = bias_std
-
-    return biases, biases_std
-
-
 def estimate_models(m, xdat, mean_theo, estimate_input):
     """estimation of modification indicators of level model"""
 
@@ -602,12 +657,6 @@ def estimate_models(m, xdat, mean_theo, estimate_input):
     # estimate_input is updated with estimated alpha, dof
     estimate_dat = estimate_effects(model_dat, estimate_input)
 
-    # estimate equation biases, given theoretical level model
-    if estimate_input["estimate_bias"]:
-        biases, biases_std = estimate_biases(model_dat, estimate_input["ymdat"])
-        estimate_dat["biases"] = biases
-        estimate_dat["biases_std"] = biases_std
-
     # set missing edges to NaN
     estimate_dat["mx_hat"][model_dat["idx"] == 0] = float("NaN")
     estimate_dat["my_hat"][model_dat["idy"] == 0] = float("NaN")
@@ -754,44 +803,6 @@ def estimate_snn(model_dat, alpha, do_print=True):
     return mx, my, sse
 
 
-def sse_bias(bias, bias_ind, model_dat, ymdat):
-    """sum of squared errors given modification indicator, Tikhonov not used"""
-    bias = bias[0]
-
-    yhat = model_dat["model"](model_dat["xdat"], bias, bias_ind)
-    ymhat = model_dat["fym"] @ yhat
-    err = ymhat - ymdat
-    sse = np.sum(err * err * diag(model_dat["selwei"]).reshape(-1, 1))
-
-    log.info("sse {:10f}, bias {:10f}".format(sse, bias))
-    return sse
-
-
-def optimize_biases(model_dat, bias_ind, ymdat):
-    """numerical optimize modification indicator for single equation"""
-
-    # optimizations parameters
-    bias_start = 0
-    method = "SLSQP"  # BFGS, SLSQP, Nelder-Mead, Powell, TNC, COBYLA, CG
-
-    log.info("\nEstimation of bias for {}:".format(model_dat["yvars"][bias_ind]))
-    out = minimize(
-        sse_bias, bias_start, args=(bias_ind, model_dat, ymdat), method=method
-    )
-
-    bias = out.x
-    sse = out.fun
-
-    if hasattr(out, "hess_inv"):
-        hess_i = inv(out.hess_inv)
-        log.info("Scalar Hessian from method {}.".format(method))
-    else:
-        hess_i = nd.Derivative(sse_bias, n=2)(bias, bias_ind, model_dat, ymdat)
-        log.info("Scalar Hessian numerically.")
-
-    return bias, hess_i, sse
-
-
 def sse_hess(mx, my, model_dat, alpha):
     """compute automatic Hessian of sse at given data and direct effects"""
 
@@ -802,12 +813,12 @@ def sse_hess(mx, my, model_dat, alpha):
     def sse_orig_vec_alg(direct):
         """computes the ad target function sum of squared errors,
         input as tensor vectors, yields Hessian in usual dimension of identified parameters"""
-        mx, my = utils.directmat(direct, model_dat["idx"], model_dat["idy"])
+        mx, my = directmat(direct, model_dat["idx"], model_dat["idy"])
         ad_model = StructuralNN(model_dat)
         ychat = ad_model(mx, my)
         return sse_orig(mx, my, fym, ychat, ydata, selwei, model_dat, alpha)
 
-    direct = utils.directvec(mx, my, model_dat["idx"], model_dat["idy"])
+    direct = directvec(mx, my, model_dat["idx"], model_dat["idy"])
     hessian = torch.autograd.functional.hessian(sse_orig_vec_alg, direct)
 
     # symmetrize Hessian, such that numerically well conditioned
@@ -817,31 +828,6 @@ def sse_hess(mx, my, model_dat, alpha):
     return hessian
 
 
-def tvals(eff, std):
-    """compute t-values by element wise division of eff and std matrices"""
-
-    assert eff.shape == std.shape
-
-    if len(eff.shape) == 1:  # vector
-        rows = eff.shape[0]
-        tvalues = np.empty(rows) * np.nan
-        for i in range(rows):
-            if std[i] != 0:
-                tvalues[i] = eff[i] / std[i]
-
-    if len(eff.shape) == 2:  # matrix
-        rows, cols = eff.shape
-        tvalues = zeros((rows, cols))
-        for i in range(rows):
-            for j in range(cols):
-                if std[i, j] != 0:
-                    tvalues[i, j] = eff[i, j] / std[i, j]
-                else:
-                    tvalues[i, j] = np.nan
-
-    return tvalues
-
-
 def compute_mediation_std(ex_hat_std, ey_hat_std, eyx, eyy, yvars, final_var):
     """compute mediation std"""
 
@@ -938,7 +924,7 @@ def total_effects_std(direct_hat, vcm_direct_hat, model_dat):
     # compare numeric and algebraic gradient of effects
     do_compare = True
     if do_compare:
-        jac_effects_num = nd.Jacobian(utils.total_from_direct)(
+        jac_effects_num = nd.Jacobian(total_from_direct)(
             direct_hat,
             model_dat["idx"],
             model_dat["idy"],
@@ -979,7 +965,7 @@ def sse_orig(mx, my, fym, ychat, ymcdat, selwei, model_dat, alpha):
     sse = sum(torch.sum(err * err * torch.diag(selwei).view(-1, 1), dim=0))
 
     # sse with tikhonov term
-    direct = utils.directvec(mx, my, model_dat["idx"], model_dat["idy"])
+    direct = directvec(mx, my, model_dat["idx"], model_dat["idy"])
     ssetikh = sse + alpha * direct.T @ direct
 
     return ssetikh.requires_grad_(True)
