@@ -1,15 +1,39 @@
 # -*- coding: utf-8 -*-
 """Create direct, total and mediation Graphviz graph from dot_str using pydot."""
-
+import locale
 from typing import Dict, Sequence, Optional
 from itertools import chain
 import numpy as np
 from numpy import allclose, isnan
 from causing import utils
-from causing.estimate import tvals
 from causing.model import Model
 
 
+def tvals(eff, std):
+    """compute t-values by element wise division of eff and std matrices"""
+
+    assert eff.shape == std.shape
+
+    if len(eff.shape) == 1:  # vector
+        rows = eff.shape[0]
+        tvalues = np.empty(rows) * np.nan
+        for i in range(rows):
+            if std[i] != 0:
+                tvalues[i] = eff[i] / std[i]
+
+    if len(eff.shape) == 2:  # matrix
+        rows, cols = eff.shape
+        tvalues = np.zeros((rows, cols))
+        for i in range(rows):
+            for j in range(cols):
+                if std[i, j] != 0:
+                    tvalues[i, j] = eff[i, j] / std[i, j]
+                else:
+                    tvalues[i, j] = np.nan
+
+    return tvalues
+
+
 def color_scheme(value, base):
     """compute colorscheme and color"""
 
@@ -89,9 +113,11 @@ def dot(
                 continue
 
             if show_in_percent:
-                wei_str = "{}{}".format(utils.roundec(100 * wei), "%")  # perc
+                wei_str = "{}{}".format(
+                    locale.format_string("%.2f", 100 * wei), "%"
+                )  # perc
             else:
-                wei_str = utils.roundec(wei)
+                wei_str = locale.format_string("%.2f", wei)
 
             col_str = specific_color_str(wei, True)
             dot_str += '         "{}" -> "{}" [label = "{}"{}];\n'.format(
@@ -108,9 +134,11 @@ def dot(
         if nodeff is not None and not isnan(nodeff[i]):
             # if no nodeff given, or some elements are nan (tval ey diag)
             if show_in_percent:
-                nodeff_str = "{}{}".format(utils.roundec(100 * nodeff[i]), "%")  # perc
+                nodeff_str = "{}{}".format(
+                    locale.format_string("%.2f", 100 * nodeff[i]), "%"
+                )  # perc
             else:
-                nodeff_str = utils.roundec(nodeff[i])
+                nodeff_str = locale.format_string("%.2f", nodeff[i])
             col_str = specific_color_str(nodeff[i], False)
         else:
             nodeff_str = ""
@@ -262,7 +290,6 @@ def create_graphs(
     direct_indiv_graphs = []
     total_indiv_graphs = []
     mediation_indiv_graphs = []
-    print()
 
     for i in range(min(show_nr_indiv, len(graph_json["mx_indivs"]))):
         if ids:
