@@ -2,7 +2,6 @@
 """Utilities."""
 
 # pylint: disable=invalid-name # spyder cannot read good-names from .pylintrc
-# pylint: disable=E1101 # "torch has nor 'DoubleTensor' menber"
 
 from typing import IO, Sequence
 from copy import deepcopy
@@ -30,7 +29,6 @@ from numpy import (
 from numpy.linalg import inv, norm
 from pandas import DataFrame
 import sympy
-import torch
 import pathlib
 
 # set numpy random seed
@@ -72,23 +70,6 @@ def nonzero(el):
     return nonz
 
 
-def roundec(num, dec=None):
-    """rounds number or string to dec decimals,
-    converts to string and strips trailing zeros and dot from the right"""
-
-    # dec
-    if not dec:
-        limit_dec = 1000  # ToDo: set limit_dec globally # yyy
-        if abs(num) < limit_dec:
-            dec = 2
-        else:
-            dec = 0
-
-    string = ("{0:." + str(dec) + "f}").format(float(num)).rstrip("0").rstrip(".")
-
-    return string
-
-
 def submatrix(mat, j):
     """computes submatrix or -vector by replacing j-th row and column by zeros"""
 
@@ -199,81 +180,6 @@ def directmat_alg(direct, idx, idy):
     return mx, my
 
 
-def directmat(direct, idx, idy):
-    """automatic direct effects matrices column-wise
-    from direct effects vector and id matrices"""
-
-    # dimensions
-    ndim = idx.shape[0]
-    mdim = idx.shape[1]
-
-    # compute direct effects matrices
-    my = torch.DoubleTensor(zeros((ndim, ndim)))
-    mx = torch.DoubleTensor(zeros((ndim, mdim)))
-    k = 0
-    for i in range(ndim):
-        for j in range(ndim):
-            if idy[i, j] == 1:
-                my[i, j] = direct[k]
-                k += 1
-    for i in range(ndim):
-        for j in range(mdim):
-            if idx[i, j] == 1:
-                mx[i, j] = direct[k]
-                k += 1
-
-    return mx, my
-
-
-def directvec_alg(mx, my):
-    """algebraic direct effects vector column-wise
-    from direct effects matrices and id matrices"""
-
-    directy = my.T[~isnan(my.T)]
-    directx = mx.T[~isnan(mx.T)]
-    direct = concatenate((directy, directx), axis=0)
-
-    return direct
-
-
-def directvec(mx, my, idx, idy):
-    """automatic direct effects vector column-wise
-    from direct effects matrices and id matrices"""
-
-    # dimensions
-    ndim = idx.shape[0]
-    mdim = idx.shape[1]
-    qydim = count_nonzero(idy)
-    qxdim = count_nonzero(idx)
-
-    # compute direct effects vector
-    direct = torch.DoubleTensor(zeros(qydim + qxdim))
-    k = 0
-    for i in range(ndim):
-        for j in range(ndim):
-            if idy[i, j] == 1:
-                direct[k] = my[i, j]
-                k += 1
-    for i in range(ndim):
-        for j in range(mdim):
-            if idx[i, j] == 1:
-                direct[k] = mx[i, j]
-                k += 1
-
-    return direct
-
-
-def total_from_direct(direct, idx, idy, edx, edy):
-    """construct total effects vector from direct effects vector and id and ed matrices"""
-
-    mx, my = directmat_alg(direct, idx, idy)
-    ex, ey = total_effects_alg(mx, my, edx, edy)
-
-    effects = directvec_alg(ex, ey)
-
-    return effects
-
-
 def digital(mat):
     """transform a matrix or vector to digital matrix,
     elements are equal to one if original element is unequal zero, and zero otherwise"""
@@ -298,6 +204,19 @@ def digital(mat):
     return mat_digital
 
 
+def print_bias(m, biases, biases_std, output_file):
+    biases = concatenate(
+        (
+            biases.reshape(1, -1),
+            biases_std.reshape(1, -1),
+            (biases / biases_std).reshape(1, -1),
+        )
+    )
+    biases_dfstr = DataFrame(biases, ("biases", "std", "t-values"), m.yvars).to_string()
+
+    print("\nbiases:\n", biases_dfstr, file=output_file)
+
+
 def print_output(
     m, xdat, estimate_dat, estimate_input, indiv_dat, mean_theo, output_file: IO
 ):
@@ -305,19 +224,6 @@ def print_output(
 
     tau = xdat.shape[1]
 
-    # compute dataframe strings for printing
-    if estimate_input["estimate_bias"]:
-        biases = concatenate(
-            (
-                estimate_dat["biases"].reshape(1, -1),
-                estimate_dat["biases_std"].reshape(1, -1),
-                (estimate_dat["biases"] / estimate_dat["biases_std"]).reshape(1, -1),
-            )
-        )
-        biases_dfstr = DataFrame(
-            biases, ("biases", "std", "t-values"), m.yvars
-        ).to_string()
-
     hessian_hat_dfstr = DataFrame(estimate_dat["hessian_hat"]).to_string()
 
     yhat = m.compute(xdat)
@@ -363,12 +269,6 @@ def print_output(
         )
     )
 
-    # biases
-    if estimate_input["estimate_bias"]:
-        pr()
-        pr("biases:")
-        pr(biases_dfstr)
-
     # algebraic direct and total effects
     # pr("\nmx_alg:")
     # pr(np.array2string(model_dat["mx_alg"]))
@@ -484,7 +384,7 @@ def acc(n1, n2):
 @np.vectorize
 def round_sig(x, sig=2) -> float:
     """Round x to the given number of significant figures"""
-    if x == 0 or isnan(x):
+    if x == 0 or not np.isfinite(x):
         return x
     return round(x, sig - int(floor(log10(abs(x)))) - 1)
 
@@ -500,7 +400,8 @@ def round_sig_recursive(x, sig=2):
         return x.__class__(round_sig_recursive(value, sig) for value in x)
     if isinstance(x, (float, np.ndarray)):
         return round_sig(x, sig)
-    if isinstance(x, torch.Tensor):
+    # avoid importing pytorch for isinstance check
+    if type(x).__name__ == "Tensor":
         return x.apply_(lambda x: round_sig(x, sig))
 
     return x
@@ -508,7 +409,8 @@ def round_sig_recursive(x, sig=2):
 
 class MatrixEncoder(json.JSONEncoder):
     def default(self, obj):
-        if isinstance(obj, (np.ndarray, torch.Tensor)):
+        # avoid importing pytorch for isinstance check
+        if isinstance(obj, np.ndarray) or type(obj).__name__ == "Tensor":
             return obj.tolist()
         return json.JSONEncoder.default(self, obj)
 
